"""
Visualisation utilities for working with alphaMELTS table outputs.

Todo
-------

    * Pull out mineral compostional trends
"""
from pyrolite.util.plot import proxy_line
from ..tables.load import import_tables
from .style import phase_color, phaseID_linestyle

from ..util.log import Handle

logger = Handle(__name__)
