from ..util.log import Handle

logger = Handle(__name__)

def get_phaselist(folder):
    """
    Get the list of phases from an experiment.

    Parameters
    -----------
    folder : :class:`str` | :class:`pathlib.Path`
        Path to the experiment directory.
    """


def get_assemblages(folder):
    """
    Get the list of phases from an experiment.

    Parameters
    -----------
    folder : :class:`str` | :class:`pathlib.Path`
        Path to the experiment directory.
    """
